new;

/*********************************************************************************************
***          Conditional Density Estimation                                               ****
***          Program by Sokbae Lee                                                        ****
*********************************************************************************************/

n_all = 1747;
d_all = 2;

et = hsec;

/********************
*** Loading Data ****
********************/


load data[n_all,d_all] = xzp.asc;
load gridz[26,1] =  z.out;

    x = data[.,1];
    z = data[.,2];

         n = rows(data);
        dd = cols(data);

   /******************************************************
   ** Bandwidth Selection by Residual Squares Criterion **
   ******************************************************/


/*************************************************************************************
*** Estimation of the Conditional PDFs of Disturbance Terms of Outcome Equations   ***
*************************************************************************************/

 tiny = 1e-8; @ lower bound for density @

   gridpx = 26;
   tmpx = sortc(x,1);
   lbcx = floor(0.01*rows(x));
    lbx = tmpx[lbcx];
   ubcx = floor(0.99*rows(x));
    ubx = tmpx[ubcx];
  udx = (ubx-lbx)/gridpx;
  usx = seqa(lbx,udx,gridpx);
  x_range = lbx~ubx;

   gridpz = 26;
   tmpz = sortc(z,1);
   lbcz = floor(0.01*rows(z));
    lbz = tmpz[lbcz];
   ubcz = floor(0.99*rows(z));
    ubz = tmpz[ubcz];
  udz = (ubz-lbz)/gridpz;
  usz = seqa(lbz,udz,gridpz);

usz = gridz;

 pdfzx = zeros(rows(usz),rows(usx));
 h2_z = 1.06*stdc(z)*(rows(z)^(-1/5));
 h2_x = 1.06*stdc(x)*(rows(x)^(-1/5));
 @h2_x = 0.5;@
 bwzx = zeros(rows(usz),1);

  j = 1;

  do until j > rows(usz);

kh2_j = kl((z - usz[j])/h2_z)/h2_z;

  i = 1;

  do until i > rows(usx);

  {mh1,dmh1} = lpre(kh2_j,x,usx[i],h2_x,1);

        tmp1 = mh1;

  pdfzx[i,j] = maxc(tmp1|tiny);

"D=1: j -- i " j~i;

   i = i + 1;
   endo;


   j = j + 1;
   endo;

output file = fzx.out reset;
output off;

format /rd /m1 2,3;

output on;
pdfzx;
output off;

output file = x.out reset;
output off;

format /rd /m1 2,3;

output on;
usx;
output off;

/*********************************************************
******                 Procedures               **********
**********************************************************/

/*************************************************/
/*************** Some Utilities ******************/
/*************************************************/

proc(1) = ftn_mu(x,p,i);
/* Inputs: x n*d data
           p order of polynomials 
           i =1 if linear interactions are included; =0 otherwise 
   Output: ftn n*(d*p + no. of interaction terms) vector 
*/  
           
local n,d,ftn,j;

  n = rows(x);
  d = cols(x);

ftn = polymat(x,p);                           @ polynomial terms @

if (i == 1);

j = 1;
do until j > (d-1);

ftn = ftn~(x[.,j].*x[.,(j+1):d]);             @ interaction terms @ 

j = j + 1;
endo;

endif;

retp(ftn);
endp;

/*************************************************/

proc(1) = grad_mu(x,p,i,w);
/* Inputs: x n*d data
           p order of polynomials 
           i =1 if linear interactions are included; =0 otherwise 
           w n*1 weights
   Output:  gr d*(d*p + no. of interaction terms) mean vector   
*/                 
           
local n,d,ftn,j,dj,cdj,gr,k,ind,xj,dx,ii;

  n = rows(x);
  d = cols(x);

ftn = polymat(x,p);                           @ polynomial terms @

if (i == 1);

j = 1;
do until j > (d-1);

ftn = ftn~(x[.,j].*x[.,(j+1):d]);             @ interaction terms @
 
j = j + 1;
endo;

endif;


 gr = {};
  k = 1;
  do until k > d;
  
      dx = zeros(n,cols(ftn));                     @ gradient @
 dx[.,k] = ones(n,1);                                             

if p > 1;  

         dx[.,seqa((d+k),d,(p-1))] = (polymat(x[.,k],p-1).*seqa(2,1,p-1));   @ derivative of polynomial terms @
endif;

if (i == 1);

 dj = seqa(d-1,-1,d-1); 
cdj = 0|cumsumc(dj);

ind = setdif(seqa(1,1,d),k,1);

 xj = x[.,ind];

if (k < d);
dx[.,(d*p + cdj[k] + 1):(d*p + cdj[k] + dj[k])] = xj[.,k:(d-1)];             @ interaction terms @
endif;

ii = 1;
do while ii < k;

dx[.,(d*p + cdj[ii] + k-ii)] = xj[.,ii];  

ii = ii + 1;
endo;

endif;

gr = gr|meanc(dx.*w)';
 
 k = k + 1;
 endo;
   
 
retp(gr);
endp;


/*************************************************/
/*************** Logit Regression ****************/
/*************************************************/

proc (5) = logit(y,x);
local cnv,b0,b1,i,ind,P,grd,hes,h1,dev,OP,EOP,se1,se2,t1,t2,ps,avar;

cnv = 0;

b0 = zeros(cols(x),1); @ starting value @

i = 1;
do until cnv == 1;

ind = x*b0;
P = exp(ind)./(1+exp(ind));

grd = sumc((y - P).*x);
hes = -x'(P.*(1-P).*x);

b1 = b0 - inv(hes)*grd;

dev = maxc(abs(b1-b0));

if dev < 1e-8; @ stopping rule @
cnv = 1;
endif;

b0 = b1;

i = i + 1;
endo;

/* One could use OP instead of EOP.
 OP = ((y-P).*x)'((y-P).*x)/n; @ outer product @
se1 = sqrt(diag(invpd(OP))/n);
 t1 = b1./se1;
*/

EOP = -hes/n;  @ expected info. @
se2 = sqrt(diag(invpd(EOP))/n);
 t2 = b1./se2;

ps = exp(x*b1)./(1+exp(x*b1));

avar = invpd(EOP);

" Logit Regression: No. of iteration " i;

retp(b1,se2,t2,ps,avar);
endp;


/*****************************************************/
/*************** Series Approximation ****************/
/*****************************************************/

proc (2) = serapp(p,basis);
@
--- Inputs ---
    p = n*1 vector of data
basis = 3*1 vector of options
        if basis[1] = 1 is chosen, basis[2]-th order polynomial approximation is used.      
        if basis[1] = 2 is chosen, basis[2]-th order P-spline approximation is used with basis[3] number of knots. 
--- Output ---
  app = n*1 vector of values of an approximating function
  dap = n*1 vector of values of the derivative of an approximating function  
@


local b1, b2, app, dap, i, b3, tmp;

 b1 = basis[1];
 b2 = basis[2];
  
app = ones(rows(p),1);
dap = {};

if b1==1;         @ polynomials @

i = 1;
do until i > b2;

app = app~(p^i);
dap = dap~(i*p^(i-1));

i = i + 1;
endo;

elseif b1==2;   @ splines @ 

b3 = basis[3];

i = 1;
do until i > b2;

app = app~(p^i);
dap = dap~(i*p^(i-1));

i = i + 1;
endo;

i = 1;
do until i > b3;

tmp = p - quantile(p,(i/(b3+1)));
tmp = (tmp .> 0).*tmp;

app = app~(tmp.^b2);
dap = dap~(b2*tmp.^(b2-1)); 

i = i + 1;
endo;

endif;

retp(app,dap);
endp;




/**********************************************************/
/*************** 2nd Order Kernel Function ****************/
/**********************************************************/
proc(1) = kl(v);
local term;
retp(pdfn(v));
endp;



/*********************************************************/
/*    Local Polynomial Regression Estimator              */
/*********************************************************/

proc(2) = lpre(y,x,x0,h,p);

@
--- Inputs ---
    y = n*1 vector of dependent variable
    x = n*1 vector of observations of independent variables
   x0 = a point where a regression function is evaluated
    h = a bandwidth
    p = order of polynomials
--- Output ---
local polynomial regression estimate (evaluated at x0)
@

local n,w,z,i,arg,b,mhat,dmhat;

   n = rows(x);

 arg = (x-x0);
   w = kl(arg/h);

   z = ones(n,1);

   i = 1;
   do until i > p;

   z = z~(arg.^i);

   i = i + 1;
   endo;

   w = w + 1e-5*(w.==0);  @ This is to avoid singularity of z'w*z. @
   w = diagrv(eye(n),w);
   b = invpd(z'w*z)*(z'w*y);

 mhat = b[1];
dmhat = b[2];
retp(mhat,dmhat);
endp;


/**********************************************/
/*    Residual Squares Criterion              */
/**********************************************/

proc(1) = rsc(y,x,x0,h,v);

@
--- Inputs ---
    y = n*1 vector of dependent variable
    x = n*1 vector of observations of indepedent variable
   x0 = a point where a regression function is evaluated
    h = a bandwidth
    v = 0 for estimation of regression function
      = k for the k-th-order derivative
--- Output ---
value of RSC
@

local p,n,arg,w0,z,i,w,tmp0,tmp1,bhat,yhat,tmp2,vmat,vm,tmp3,tmp4,tmp5,sm2,rsc;

   p = v + 1;
   n = rows(x);

 arg = (x-x0);

  w0 = kl(arg/h);

   z = ones(n,1);

   i = 1;
   do until i > p;

   z = z~(arg.^i);

   i = i + 1;
   endo;

   w = w0 + 1e-5*(w0.==0);  @ This is to avoid singularity of z'w*z. @
   w = diagrv(eye(n),w);
tmp0 = z'w*y;
tmp1 = invpd(z'w*z);
bhat = tmp1*tmp0;
yhat = z*bhat;
tmp2 = z'(w^2)*z;
vmat = tmp1*tmp2*tmp1;
  vm = vmat[1,1];

tmp3 = w - w*z*tmp1*z'w;
tmp4 = sumc(diag(tmp3));          @ trace of tmp3 @
tmp4 = tmp4 + 1e-8*(tmp4.==0);
tmp5 = sumc(((y-yhat)^2).*w0);
 sm2 = tmp5/tmp4;

 rsc = sm2*(1+(p+1)*vm);

retp(rsc);

endp;


/******************************************************
** Bandwidth Selection by Residual Squares Criterion **
******************************************************/

proc(1) = bdwrsc(y,x,x_range,v);
@
--- Inputs ---
    y = n*1 vector of dependent variable
    x = n*1 vector of observations of indepedent variable
    x_range = 2*1 vector [xmin, xmax]: range where a regression function is evaluated
    v = 0 for estimation of regression function
      = k for the k-th-order derivative (maximum k is 3).
--- Output ---
optimal value of RSC
@
local xmin,xmax,dx,xseq,hmin,hmax,h_cons,rscmat,bdwmat,h_ind,k,bdw,irsc,j,h_rsc,bdw_opt;


xmin = x_range[1];
xmax = x_range[2];
  dx = (xmax-xmin)/10;
xseq = seqa(xmin,dx,11);


  hmin = (maxc(x) - minc(x))/20;
  hmax = (maxc(x) - minc(x))/2;
  h_cons = 1.1;

  rscmat = {};
  bdwmat = {};

   h_ind = 0;

k = 1;
do until (h_ind == 1);

bdw = hmin*(h_cons^k);


irsc = {};
j = 1;
do until j > rows(xseq);

  h_rsc = rsc(y,x,xseq[j],bdw,v);
   irsc = irsc|h_rsc;

j = j + 1;
endo;

irsc = sumc(irsc);

" Bandwidth -- IRSC " bdw~irsc;

rscmat = rscmat|irsc;
bdwmat = bdwmat|bdw;

h_ind = (bdw > hmax);

k = k + 1;
endo;

bdw_opt = bdwmat[minindc(rscmat)];

bdw_opt = (v==0)*bdw_opt + (v==1)*0.8403*bdw_opt + (v==2)*0.8285*bdw_opt + (v==3)*0.8085*bdw_opt;

retp(bdw_opt);
endp;


/**********************************************/
/*   Asymptotic Variance Estimation           */
/**********************************************/

proc(1) = ave(y,x,x0,h,hs,v);

@
--- Inputs ---
    y = n*1 vector of dependent variable
    x = n*1 vector of observations of independent variable
   x0 = a point where a regression function is evaluated
    h = a bandwidth that is used for estimation of the function
   hs = a pilot bandwidth
    v = 0 for estimation of regression function
      = 1 for the first-order derivative
--- Output ---
asymptotic variance estimate
@

local p,pa,n,arg,w0,z,i,w,tmp0,tmp1,bhat,yhat,tmp2,vmat,vm,tmp3,tmp4,tmp5,sm2,vc;

   p = v + 1;
  pa = p + 2;
   n = rows(x);

 arg = (x-x0);
  w0 = kl(arg/hs);
   z = ones(n,1);

   i = 1;
   do until i > pa;

   z = z~(arg.^i);

   i = i + 1;
   endo;

   w = w0 + 1e-5*(w0.==0);  @ This is to avoid singularity of z'w*z. @
   w = diagrv(eye(n),w);
tmp0 = z'w*y;
tmp1 = invpd(z'w*z);
bhat = tmp1*tmp0;
yhat = z*bhat;
tmp2 = z'(w^2)*z;
tmp3 = w - w*z*tmp1*z'w;
tmp4 = sumc(diag(tmp3));          @ trace of tmp3 @
tmp4 = tmp4 + 1e-8*(tmp4.==0);
tmp5 = sumc(((y-yhat)^2).*w0);
 sm2 = tmp5/tmp4;

 arg = (x-x0);
  w0 = kl(arg/h);
   z = ones(n,1);

   i = 1;
   do until i > p;

   z = z~(arg.^i);

   i = i + 1;
   endo;

   w = w0 + 1e-5*(w0.==0);  @ This is to avoid singularity of z'w*z. @
   w = diagrv(eye(n),w);

tmp1 = invpd(z'w*z);
tmp2 = z'(w^2)*z;
vmat = (tmp1*tmp2*tmp1)*sm2;
  vc = vmat[v+1,v+1];

retp(vc);

endp;
/*******************************************/
/**** CV Function for Series Estimation ****/
/*******************************************/
proc(1) = cv_ser(y,x,p,basis);

@
--- Inputs ---
    y = n*1 vector of dependent variable
    x = n*d vector of observations of covariates (parametric)
    p = n*1 vector of observations of a covariate (nonparametric) 
    h = 3*1 vector of options
--- Output ---
value of cv function at h
@

local n,i,temp,ei,yi,xi,fi,f,df,reg,bhat,yhat,cvh;

      n = rows(y);
{f, df} = serapp(p,basis);

temp = {};
   i = 1;
   do until i > n;

    ei = zeros(n,1);
 ei[i] = 1;
    yi = delif(y,ei);
    xi = delif(x,ei);
    fi = delif(f,ei);
    reg = xi~fi;
   bhat = invpd(reg'reg)*(reg'yi);
   yhat = (x[i,.]~f[i,.])*bhat;

  temp = temp|yhat;

   i = i + 1;
   endo;

    cvh = meanc((y-temp)^2);
 
retp(cvh);
endp;

/*******************************************************/
/*  Numerical Integration: composite trapezoidal rule  */
/*******************************************************/

proc(1) = ctr(dta,diff);
local tmp;
tmp = sumc(dta) - 0.5*dta[1,.]' - 0.5*dta[rows(dta),.]';
tmp = tmp*diff;
retp(tmp);
endp;

   /*********************************************************/
   /*             Kernel Estimator                          */
   /*********************************************************/

proc(1) = nwk(y,x,x_arg,h);

@
--- Inputs ---
    y = n*1 vector of dependent variable
    x = n*d vector of observations of independent variables
x_arg = k*d vector of arguments of regression function
    h = d*1 vector of bandwidths
--- Output ---
k*1 vector of kernel estimates
@

local n,d,k,i,arg,tmp1,tmp2,m,mhat;

   n = rows(x);
   d = cols(x);
   k = rows(x_arg);

tmp1 = ones(k,n);
   i = 1;
   do until i > d;

          arg = (x_arg[.,i]-x[.,i]')/h[i];
         tmp1 = tmp1.*kl(arg);

   i = i + 1;
   endo;

   m = y';
tmp2 = tmp1.*m;
mhat = sumc(tmp2')./(1e-8 + sumc(tmp1'));

retp(mhat);
endp;
