%treatment parameters

clear all

supportlow = 0.099;
supporthigh = 0.932;

%density of z and x
load fzx.out;
load x.out;
load z.out;
auxszx = size(fzx);
auxzx = [];
for i = 1:2:auxszx(1)-1;
    auxzx = [auxzx;fzx(i,:) fzx(i+1,:)];
end;

fzx = auxzx'; 
fz = find(or(z<supportlow,z>supporthigh));
lx = length(x);

for i = 1:lx
	fzx(fz,i)=0;
end;
aux = sum(fzx);
for i = 1:length(aux);
    fzx(:,i)=fzx(:,i)/aux(i);
end;

%get mte

pdom = x;
zdom = z;
lpdom = length(pdom);
lzdom = length(zdom);

load xzp.asc
xindex = xzp(:,1);

load xbeta.asc;
xb = xbeta(:,2);

mtepaux = ksr_p(xindex,xb,pdom,(max(xindex)-min(xindex))/10);
mtep = mtepaux.f;

load mte.out;

u = mte(:,1);
lu = length(u);
mteu = mte(:,2);

%construct MTE(P,U)

mtepu = zeros(lu,lpdom);
for i = 1:lu
    for j = 1:lpdom
        mtepu(i,j) = mteu(i)+mtep(j);
    end;
end;

%get weights

%Densities of X,U,Z


minxb = min(xindex);
maxxb = max(xindex);
mxb = maxxb - minxb;
xdom = pdom;
fx = kerndens(xindex,xdom,mxb/10);
fx = fx/sum(fx);

%ATE

atew = zeros(lu,lpdom);
for i = 1:lu
    for j = 1:lpdom
        atew(i,j) = fx(j)*(1/lu);
    end;
end;

%TT

ttw = zeros(lu,lpdom);
ttwz = zeros(lu,lpdom);
for i = 1:lu
    f1 = find(zdom<=u(i));
    for j = 1:lpdom
        ttwz(i,j) = sum(fzx(length(f1)+1:length(zdom),j));
    end;
end;
aux=sum(ttwz);
for j = 1:lpdom
    ttw(:,j) = (ttwz(:,j)/aux(j))*fx(j);
end;

%TUT

tutw = zeros(lu,lpdom);
tutwz = zeros(lu,lpdom);
for i = 1:lu
    f1 = find(zdom<=u(i));
    for j = 1:lpdom
        tutwz(i,j) = sum(fzx(1:length(f1),j));
    end;
end;
aux=sum(tutwz);
for j = 1:lpdom
    tutw(:,j) = (tutwz(:,j)/aux(j))*fx(j);
end;

%Policy - P + alpha

ttwpol = zeros(lu,lpdom);
for j = 1:lpdom
    ttwpol(:,j) = fzx(:,j)*fx(j);
end;

%Policy - (1+alpha)*P

ttwpolb = zeros(lu,lpdom);
ttwzpolb = zeros(lu,lpdom);
for i = 1:lu
    ttwzpolb(i,:) = fzx(i,:)*u(i);
end;
aux=sum(ttwzpolb);
for j = 1:lpdom
    ttwpolb(:,j) = (ttwzpolb(:,j)/aux(j))*fx(j);
end;

%Policy = Tuition - alpha

ttwpolc = zeros(lu,lpdom);
ttwzpolc = zeros(lu,lpdom);
auxu = log(u./(1-(u-0.000000001)));
auxuu = exp(auxu)./((1+exp(auxu)).^2);
auxuuu = auxuu/(sum(auxuu));
for i = 1:lu
    ttwzpolc(i,:) = fzx(i,:)*auxuuu(i);
end;
aux=sum(ttwzpolc);
for j = 1:lpdom
    ttwpolc(:,j) = (ttwzpolc(:,j)/aux(j))*fx(j);
end;


%parameters

ate = sum(sum(mtepu.*atew));
tt = sum(sum(mtepu.*ttw));
tut = sum(sum(mtepu.*tutw));
policy = sum(sum(mtepu.*ttwpol));
policyb = sum(sum(mtepu.*ttwpolb));
policyc = sum(sum(mtepu.*ttwpolc));

treat = [ate tt tut policy policyb policyc];

save treat.out treat -ASCII

j = 12
[ttwpol(:,j)'*mtepu(:,j) ttwpolb(:,j)'*mtepu(:,j) ttwpolc(:,j)'*mtepu(:,j)]/(4*fx(j))
[ttwpol(:,j) ttwpolb(:,j) ttwpolc(:,j)]/fx(j)

fs = find(and(u>=supportlow,u<=supporthigh));

ufs = u(fs);
smte = mtepu(fs,j)/4;
aux1 = ttwpol(fs,j)/fx(j);
aux2 = mean(smte)/mean(aux1);
spol = aux2*ttwpol(:,j)/fx(j);
spolb = aux2*ttwpolb(:,j)/fx(j);
spolc = aux2*ttwpolc(:,j)/fx(j);

figure(3)
plot(ufs,smte,u,spol,'k-o',u,spolc,'k--',u,spolb,'k-d');
xlim([0,1]);
legend('MTE','MPRTE: Pa = P0+a; AMTE: |P-U|<e','MPRTE: Pa = Fv(Zg+ag); AMTE: |ZGamma-V|<e','MPRTE: Pa = (1+a)P; AMTE: |(P/U)-1|<e');
title('Alternative Definitions of the MPRTE and the AMTE');
xlabel('U');
ylabel('MTE; MPRTE and AMTE Weights');
print -dpsc weights3.ps
